function [homog_factor, inlier_idx, residuum] = class_homogenity(data_mat, thresh, idx_est, idx_verify)
    
% [homog_factor, inlier_idx, residuum] = is_homogenious(data, thresh, idx_estimation[, idx_verification=idx_estimation])
%
% return how homogenious a class is (0 worst, 1 best)
%
% residuum is a row vector containing the squared residuum (for all
% non-idx values, it is set to NaN)
    
    if nargin < 4
        idx_verify = idx_est;
    end

    if numel(idx_est) < 4
        homog_factor = 0;
        inlier_idx = [];
        residuum = [];
        return
    end
    
    H = estimate_H(data_mat(idx_est, :), thresh);
    if cond(H) > 1000000
        homog_factor = 0;
        inlier_idx = [];
        residuum = [];
        return
    end

    HI = inv(H);

    d12 = symerr(data_mat(idx_verify, :), H, HI);
    inlier_idx = find(d12 < thresh);

    homog_factor = numel(inlier_idx) / numel(idx_verify);

    %residuum = zeros(1, size(data_mat, 1)) * NaN;
    %residuum(idx) = d12.^2;
    residuum = d12.^2;
    
    