function [inlier_idx_test, residuum] = class_inlier(data_mat, thresh, idx_class, idx_test)
    
% [homog_factor, inlier_idx, residuum] = is_homogenious(data, thresh, idx)
%
% return how homogenious a class is (0 worst, 1 best)
%
% residuum is a row vector containing the squared residuum (for all
% non-idx values, it is set to NaN)
    
    if numel(idx_class) < 4
        inlier_idx_test = [];
        residuum = [];
        return
    end
    
    H = estimate_H(data_mat(idx_class, :), thresh);
    if cond(H) > 1000000
        inlier_idx_test = [];
        residuum = [];
        return
    end

    HI = inv(H);

    d12_test = symerr(data_mat(idx_test, :), H, HI);
    inlier_idx_test = find(d12_test < thresh);

    d12_class = symerr(data_mat(idx_class, :), H, HI);
    %inlier_idx_class = find(d12_class < thresh);
    
    residuum = zeros(1, size(data_mat, 1)) * NaN;
    residuum(idx_test) = d12_test.^2;
    residuum(idx_class) = d12_class.^2;
    