function [idx,global_residual_mat,row_labels] = msmc(trajs, K, scales, debug)

% [idx,global_residual_mat] = msmc(trajs, K, scales, debug)
% trajs is a 3xn_trajsxn_frames matrix, where nan denotes that data is
% missing

if nargin < 4
    debug=0;
end

assert(size(trajs,1) == 3);
n_trajs=size(trajs,2);
n_frames=size(trajs,3);
n_scales=numel(scales);

%%
global_residual_mat=zeros(0, n_trajs);
row_labels=cell(0);
for f1=1:n_frames
    for s=1:n_scales
        f2=f1+s;
        if f2 > n_frames
            continue;
        end
        
        data_mat = [trajs(:,:,f1)', trajs(:,:,f2)'];
        [known_data,dc]=find(~isnan(data_mat));
        known_data=unique(known_data);
        [idx, residual_mat] = msam(data_mat(known_data, :));
        n_motions = size(residual_mat,1);
        global_residual_mat(size(global_residual_mat,1)+(1:n_motions), ...
            known_data) = residual_mat;
        for i=1:n_motions
            row_labels{numel(row_labels)+1} = sprintf('f %d, s %d, m %d', f1, s, i);
        end
    end
end

%%
J=squareform(pdist(global_residual_mat', 'jaccard'));
A=1-J;
A(isnan(A))=0;
[dc,idx,errorsum,dc,C]= sc_sym(A, K, 'kmeans');

%%
if debug
    plot_segmentation2 (global_residual_mat, 1:n_trajs, idx, ...
                       row_labels, 0.5, A,  1:n_frames, trajs, 'tmp')
end
