function plot_classes(class_labels, data, I)

% plot_classes(class_labels, data[, I])
%
% plot the spatial distribution of the destination part of data, given
% the classification class_labels

assert(numel(class_labels) == size(data, 1));
switch size(data, 2)
 case 2 % x y
  data = [data, ones(size(data, 1), 1)];
 case 3 % x y h
  % do nothing 
 case 4 % x1 y1 x2 y2
  data = [data(:,3:4) ones(size(data, 1), 1)];
 case 6 % x1 y1 h1 x2 y2 h2
  data = data(:,4:6);
 otherwise
  assert(false);
end

msam_globals

info_mode=0;
n_motions = max(class_labels);

x=data(:,1) / data(:,3);
y=data(:,2) / data(:,3);
valid_motions = unique(class_labels);
valid_motions = reshape(valid_motions, 1, []);
    

if nargin >= 3
  axis auto
  if size(I,3) == 1
    I = cat(3, I, I, I);
  end
  image(I);
  axis manual
  hold on
else
  axis ij;
  hold on
end

class_legend=cell(0);
n_plots=0;
for i = 1:numel(valid_motions)
    class=valid_motions(i);
    
    idx_class = find(class_labels == class);
    if isempty(idx_class) 
      continue
    end
  
    n_plots=n_plots+1;
    class_legend{n_plots} = sprintf('class %d (%d)', class, ...
                                    numel(idx_class));
    drawlabels=class_drawlabel(mod(class-1, n_drawlabels)+1,:);
    drawcolors=class_color(mod(class-1, n_drawlabels)+1,:);
    plot(x(idx_class), y(idx_class), drawlabels, 'color', drawcolors, ...
         'MarkerSize', marker_size, 'LineWidth', marker_width);
end

if isempty(class_legend)
    legend('No points detected in this frame');
else
    legend(class_legend);
end
if info_mode
  title('Classified points');
  xlabel('x');
  ylabel('y');
else
  axis off
end

hold off