function plot_residuum_mat(class_label, residuum_mat, ids, rm_thresh, row_labels)

% plot_residuum_mat(class_label, residuum_mat)
%
% Plot the linear residuum matrix, sorted by class class_label.
    
    n_elements = numel(class_label);
    if nargin < 3
        ids = [];
    end
    if nargin < 4
      rm_thresh = [];
    end
    if nargin < 5
      row_labels = [];
    end
    
    assert(size(residuum_mat, 2) == n_elements);
    
        
    msam_globals
        
    valid_motions = unique(class_label);
    valid_motions = reshape(valid_motions, 1, []);
    n_samples = numel(find(class_label>0));
    
    order=zeros(1, n_samples);
    
    j=1;
    for i=1:numel(valid_motions)
        class=valid_motions(i);
        
        this_motion = find(class_label == class);
        n_this_motion = numel(this_motion);
        next_j = j + n_this_motion;
        bar(i).l=j;
        bar(i).r=next_j-1;
        bar(i).class=class;
        
        order(j:next_j-1) = this_motion;
        j=next_j;
    end
    
    %%
    residuum_ordered = residuum_mat(:, order);
    if isempty(rm_thresh)
      I=-log(residuum_ordered);
    else
      I=double(residuum_ordered < rm_thresh);
    end
    I(isnan(residuum_ordered)) = nan;
    imagesc(I);
    colorbar;
    colormap(gray);
    title('residuum');
    xlabel('i');
    ylabel('h');
    if (~isempty(ids))
      set(gca,'XTick', 1:n_elements);
      set(gca,'XTickLabel', ids(order));
    end
    if ~isempty(row_labels)
      set(gca,'YTick', 1:size(residuum_mat,1));
      set(gca,'YTickLabel', row_labels);
    end      
    
    hold on
    J = create_color_from_nan(I);
    image(J);

    axis_limits=axis;
    for i=bar
        xmin=i.l;
        xmax=i.r;
        xmean=mean([xmax, xmin]);
        y=-5;
        
        drawlabel=class_drawlabel(mod(i.class-1, n_drawlabels)+1, :);
        drawcolor=class_color(mod(i.class-1, n_drawlabels)+1, :);
        plot([xmin xmin], axis_limits(3:4), [drawlabel '-'],'LineWidth',2,'Color', drawcolor);
        plot([xmax xmax], axis_limits(3:4), [drawlabel '-'],'LineWidth',2,'Color', drawcolor);
        text(xmean, 0, num2str(i.class), 'color', drawcolor);
    end
    hold off
    
    
    