function plot_segmentation (sq_residuum_mat, traj_ids, class_label, ...
                            row_labels, outl_thresh, d, ...
                            all_imgs, all_corrs, map_kp_to_col, savedir)
    
    images = find(~cellfun(@isempty,all_imgs));
    n_classes = max(class_label);
    
    %% draw residuum
    choose_figure(9)
    clf
    plot_residuum_mat(class_label, sqrt(sq_residuum_mat), traj_ids, outl_thresh, row_labels);
    if exist('savedir', 'var')
        outfile=sprintf('%s/cluster-residuum.eps', savedir);
        exportfig(gcf, outfile);
    end

    %% draw clustering
    choose_figure(11);
    clf;
    plot_segmentation_result(d, class_label, traj_ids);
    if exist('savedir', 'var')
        outfile=sprintf('%s/cluster-segmentation.eps', savedir);
        exportfig(gcf, outfile);
    end
    
    %% Draw in images
    for imgnr=images    

        I = all_imgs{imgnr};
        these_corr_ids = all_corrs{imgnr, 1}.ids;
        data_mat = all_corrs{imgnr, 1}.data_mat;
        these_orig_ids = these_corr_ids(:, 1);

        %% get the traj ids 
        traj_ids = map(these_orig_ids, map_kp_to_col);
        valid_traj_ids = ~isnan(traj_ids);
        assert(size(data_mat, 1) == numel(traj_ids))
        
        %% get the valid data
        %assert(numel(idx) == size(these_corr_ids, 1)); there could be outliers
        this_class_label = class_label(traj_ids(valid_traj_ids));
        this_data = data_mat(valid_traj_ids, :);
        this_ts = all_corrs{imgnr, 1}.ts;

        choose_figure(10);
        clf;
        plot_classes(this_class_label, this_data, I);
        title(sprintf('frame %d', imgnr));

        %% create output
        this_corrs.data_mat = this_data;
        this_corrs.ts = this_ts(valid_traj_ids, :);
        this_corrs.ids = these_corr_ids(valid_traj_ids, :);
        for class = 1:n_classes 
            this_class_idx = find(this_class_label == class);
            if exist('savedir', 'var')
                outfile=sprintf('%s/img-%d_cluster-%d.corr', savedir, imgnr, class);
                dump_correspondences(this_corrs, outfile, this_class_idx);
            end
        end
        if exist('savedir', 'var')
            outfile=sprintf('%s/cluster-%d-%08d.png', savedir, n_classes, imgnr);
            print('-dpng', '-r100', outfile)
        else
            pause
        end

    end

