function plot_segmentation_result(d, class_label, ids)

  % plot_segmentation_result(d, class_label)
  %
  % Plot the distance/similarity matrix d given the classification class_label.

    n_elements = numel(class_label);
    if nargin < 3
        ids = 1:n_elements;
    end
    
    assert(size(d, 1) == n_elements);
    assert(size(d, 1) == size(d, 2));
  
    msam_globals

    distance_matrix = d(1,1) == 0;
    class_label = reshape(class_label, n_elements, 1);
    
    upscale_size=1000;
    scalefactor = upscale_size / n_elements;
    
    [sorted_class_label, order] = sort(class_label);

    %colormap(gray);
    %imagesc( d(order, order) );
    J = create_color_from_nan(d(order, order));
    J = imresize(J, scalefactor, 'Nearest');
    image(J);
    if distance_matrix
        title('d(i,j)');
    else
        title('sim(i,j)');
    end
    axis equal;
    axis([0, upscale_size, 0, upscale_size]);
    xlabel('i');
    ylabel('j');
    if ~isempty(ids) && numel(ids) < 40
      if numel(ids) < 10
        set(gca,'XTick', (0.5:n_elements) * scalefactor)
        set(gca,'XTickLabel', ids(order))
      else
        set(gca,'XTick', [])
      end
      set(gca,'YTick', (0.5:n_elements) * scalefactor)
      set(gca,'YTickLabel', ids(order))
    else
      axis off
    end
    
    N = size(d, 1);
    class_borders=[0; find(diff(sorted_class_label) > 0); N];
    valid_motions = unique(sorted_class_label);
    valid_motions = reshape(valid_motions, 1, []);
    
    hold on
    shift=.5;
    for i=1:numel(valid_motions)
        class = valid_motions(i);
        
        xmin=scalefactor*(class_borders(i))+shift;
        xmax=scalefactor*(class_borders(i+1))+shift;
        xmean=mean([xmin, xmax]);
        color=class_color(mod(class-1, n_drawlabels)+1, :);
        
        drawlabel=class_drawlabel(mod(class-1, n_drawlabels)+1, :);
        plot([xmin xmax xmax xmin xmin], [xmin xmin xmax xmax xmin], '-', 'color', color);
        
        text(xmean, xmean, num2str(class), 'color', color);
        
    end
    %colorbar;
    hold off
