function [class_label] = split_and_merge_clusters(data_mat, class_label, thresh)

% go through the classes. First, we split the classes and then we
% merge. 

msam_globals;




%% split classes
possible_classes = unique(class_label);
max_class_id = max(possible_classes);
for i=possible_classes
    %%
    if i==0
        % outlier class
        continue
    end
    
    i_idx = find(class_label == i);
    n_i_idx = numel(i_idx);
    if n_i_idx <= min_class_size
       fprintf('Dissolving class %d\n', i);

        class_label(i_idx) = 0;
        continue
    end
    
    %%
    [homogenity, i_idx_inlier] = class_homogenity(data_mat, thresh, i_idx);
    if homogenity < homog_factor_remove_outlier
      if homogenity < homog_factor_split
          
        % split i into two parts
        dissolve_class = n_i_idx < min_class_size;
        if ~dissolve_class
        
            residuum_mat = resample_idx(data_mat, i_idx, 10);
            
            [i_idx_idx] = cluster_samples2( double(sqrt(residuum_mat)), data_mat(i_idx,:), thresh, 2, ...
                                 method);
                             
            if isempty(i_idx_idx)
                % clustering failed
                dissolve_class = true;
            else
                first_class = i_idx_idx == 1;
                second_class = i_idx_idx == 2;
                if nnz(first_class < min_class_size)
                    new_class_id1=0;
                else
                    new_class_id1=i;
                end
                if (nnz(second_class) < min_class_size)                    
                    new_class_id2=0;
                else
                    max_class_id = max_class_id + 1;
                    new_class_id2 = max_class_id;
                end
                % first part is old class id i
                % second part is max_class_id
                fprintf('Splitting class %d into %d and %d\n', ...
                        i, new_class_id1, new_class_id2);
                class_label(i_idx(first_class)) = new_class_id1;
                class_label(i_idx(second_class)) = new_class_id2;
            end
        end
            
        if dissolve_class
          fprintf('Dissolving class %d\n', i);

          class_label(i_idx) = 0;
        end
      else
          
        % remove outliers
        fprintf('Removing %d outliers from class %d\n', ...
                numel(i_idx) - numel(i_idx_inlier), i);

        class_label(i_idx) = 0;
        class_label(i_idx(i_idx_inlier)) = i;
      end
    end
    
end


%% merge classes
possible_classes = unique(class_label);
max_class_id = max(possible_classes);
if possible_classes(1) == 0
    possible_classes = possible_classes(2:end);
end
for i=possible_classes
    % bigger
    i_idx = find(class_label == i);
    if numel(i_idx) == 0
        continue
    end

    for j=possible_classes
        %smaller
        j_idx = find(class_label == j);
        if numel(j_idx) >= numel(i_idx)
            continue
        end
        
        [homogenity] = class_homogenity(data_mat, thresh, i_idx, j_idx);
        if homogenity > homog_factor_merge
            fprintf('Homogenity=%f: Merging classes %d and %d\n', homogenity, i, j);
        
            % first part is old class id i
            % second part is max_class_id
            class_label(j_idx) = i;
            possible_classes = setdiff(possible_classes, j);
            %pause
        end
    end 
end


%% split outliers
j_idx = find(class_label == 0);
max_class_id = max(class_label);
for i=1:10
    if numel(j_idx) <= min_class_size
        break
    end
    
	[dc, inlier_idx] = class_homogenity(data_mat, thresh, j_idx);

    n_inliers = numel(inlier_idx);
    if n_inliers >= min_class_size
        max_class_id = max_class_id + 1;
        class_label(j_idx(inlier_idx)) = max_class_id;
        j_idx = find(class_label == 0);
    end
end


%% merge outliers
possible_classes = unique(class_label);
if possible_classes(1) == 0
    possible_classes = possible_classes(2:end);
end
j_idx = find(class_label == 0);
for i=possible_classes
    
    i_idx = find(class_label == i);

    [inlier_idx_j] = class_inlier(data_mat, thresh, i_idx, j_idx);
    n_inliers = numel(inlier_idx_j);
    if n_inliers > 0
        fprintf('Adding %d outliers to class %d\n', n_inliers, i);
        
        class_label(j_idx(inlier_idx_j)) = i;
        j_idx = find(class_label == 0);
    end
end

